@@ -34,12 +34,19 @@
 #import	<GNUstepBase/GSConfig.h>
 
 #if     defined(_NATIVE_OBJC_EXCEPTIONS)
-#define USER_NATIVE_OBJC_EXCEPTIONS       1
+#  define USER_NATIVE_OBJC_EXCEPTIONS       1
+#elif defined(BASE_NATIVE_OBJC_EXCEPTIONS) && defined(OBJC_ZEROCOST_EXCEPTIONS)
+#  define USER_NATIVE_OBJC_EXCEPTIONS       1
+#  define _NATIVE_OBJC_EXCEPTIONS           1
+#endif
+
+#if     !BASE_NATIVE_OBJC_EXCEPTIONS && USER_NATIVE_OBJC_EXCEPTIONS
+#error "There are two separate exception handling mechanisms available ... one based on the standard setjmp() function (which does not require special compiler support), and one 'native' version where the compiler manages the exception handling.  If you try to use both in the same executable, exception handlers will not work... which can be pretty disastrous.  This error is telling you that the gnustep-base library was built using one form of exception handling, but that the gnustep-make package you are using is building code to use the other form of exception handling ... with the consequence that exception handling would be broken in the program you are building.  So, somehow your gnustep-base and gnustep-make package are incompatible, and you need to replace one of them with a version configured to match the other."
+#if     BASE_NATIVE_OBJC_EXCEPTIONS
+#error  "gnustep-base is configured to use 'native' exceptions, but you are building for 'traditional' exceptions."
 #else
-#define USER_NATIVE_OBJC_EXCEPTIONS       0
+#error  "gnustep-base is configured to use 'traditional' exceptions, but you are building for 'native' exceptions."
 #endif
-#if     BASE_NATIVE_OBJC_EXCEPTIONS != USER_NATIVE_OBJC_EXCEPTIONS
-#error  The current setting for native-objc-exceptions does not match that of gnustep-base ... please correct this.
 #endif
 
 #import	<Foundation/NSString.h>
@@ -117,7 +124,7 @@ extern "C" {
  * <em>raised</em> using the -raise method.
  */
 + (void) raise: (NSString*)name
-	format: (NSString*)format,...;
+	format: (NSString*)format,... NS_FORMAT_FUNCTION(2,3);
 
 /**
  * Creates an exception with a name and a reason string using the
@@ -127,9 +134,9 @@ extern "C" {
  */
 + (void) raise: (NSString*)name
 	format: (NSString*)format
-     arguments: (va_list)argList;
+     arguments: (va_list)argList NS_FORMAT_FUNCTION(2,0);
 
-#if OS_API_VERSION(100500,GS_API_LATEST) && GS_API_VERSION( 11501,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5,GS_API_LATEST) && GS_API_VERSION( 11501,GS_API_LATEST)
 /** Returns an array of the call stack return addresses at the point when
  * the exception was raised.  Re-raising the exception does not change
  * this value.
@@ -137,7 +144,7 @@ extern "C" {
 - (NSArray*) callStackReturnAddresses;
 #endif
 
-#if OS_API_VERSION(100600,GS_API_LATEST) && GS_API_VERSION( 11903,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6,GS_API_LATEST) && GS_API_VERSION( 11903,GS_API_LATEST)
 /**
  * Returns an array of the symbolic names of the call stack return addresses.  
  * Note that, on some platforms, symbols are only exported in
@@ -263,7 +270,7 @@ typedef struct _NSHandler
 {
     jmp_buf jumpState;			/* place to longjmp to */
     struct _NSHandler *next;		/* ptr to next handler */
-    NSException *exception;
+    __unsafe_unretained NSException *exception;
 } NSHandler;
 
 /**
@@ -328,7 +335,7 @@ NSSetUncaughtExceptionHandler(NSUncaughtExceptionHandler *handler);
    It is illegal to exit the first block of code by any other means than
    NS_VALRETURN, NS_VOIDRETURN, or just falling out the bottom.
  */
-#if     defined(_NATIVE_OBJC_EXCEPTIONS)
+#if     USER_NATIVE_OBJC_EXCEPTIONS
 
 # define NS_DURING       @try {
 # define NS_HANDLER      } @catch (NSException * localException) {
@@ -338,7 +345,7 @@ NSSetUncaughtExceptionHandler(NSUncaughtExceptionHandler *handler);
 # define NS_VALUERETURN(object, id)     return (object)
 # define NS_VOIDRETURN                  return
 
-#else // _NATIVE_OBJC_EXCEPTIONS
+#elif   !USER_NATIVE_OBJC_EXCEPTIONS && !BASE_NATIVE_OBJC_EXCEPTIONS
 
 /** Private support routine.  Do not call directly. */
 GS_EXPORT void _NSAddHandler( NSHandler *handler );
@@ -350,8 +357,8 @@ GS_EXPORT void _NSRemoveHandler( NSHandler *handler );
 		    if( !setjmp(NSLocalHandler.jumpState) ) {
 
 #define NS_HANDLER _NSRemoveHandler(&NSLocalHandler); } else { \
-		    NSException *localException;               \
-		    localException = NSLocalHandler.exception; \
+		    NSException __attribute__((unused)) *localException \
+		      = NSLocalHandler.exception; \
 		    {
 
 #define NS_ENDHANDLER }}}
